import logging
import sys

from PySide6 import QtWidgets
from PySide6.QtWidgets import QDialog, QMessageBox

from utils.config import UserConfig, TransactionLevel

class SettingsDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.logger = logging.getLogger(__name__)
        self.user_config = UserConfig()

        self.setWindowTitle("Settings")
        self.setMinimumSize(400, 100)

        # Create layout
        layout = QtWidgets.QVBoxLayout(self)

        data_mode_layout = QtWidgets.QHBoxLayout()

        # Transaction isolation level
        self.data_mode_label = QtWidgets.QLabel(UserConfig.get_friendly_name("transaction_level") + ":")
        data_mode_layout.addWidget(self.data_mode_label)

        self.data_mode_dropdown = QtWidgets.QComboBox()
        for tl in TransactionLevel:
            self.data_mode_dropdown.addItem(tl.name.capitalize(), tl)
        self.data_mode_dropdown.setCurrentIndex(
            list(TransactionLevel).index(self.user_config.transaction_level)
        )

        data_mode_layout.addWidget(self.data_mode_dropdown)

        # Slowdown simulation
        self.slowdown_label = QtWidgets.QLabel(UserConfig.get_friendly_name("simulate_slowdown") + ":")
        data_mode_layout.addWidget(self.slowdown_label)

        self.slowdown_checkbox = QtWidgets.QCheckBox()
        self.slowdown_checkbox.setChecked(self.user_config.simulate_slowdown)

        data_mode_layout.addWidget(self.slowdown_checkbox)

        layout.addLayout(data_mode_layout)

        # Set the currently selected mode to the mode in UserConfig
        config = UserConfig()

        # Transaction level
        current_level = config.transaction_level
        index = self.data_mode_dropdown.findData(current_level)
        if index != -1:
            self.data_mode_dropdown.setCurrentIndex(index)

        # Slowdown simulation
        simulate_slowdown = config.simulate_slowdown
        self.slowdown_checkbox.setChecked(simulate_slowdown)

        # Buttons
        button_layout = QtWidgets.QHBoxLayout()
        
        self.save_button = QtWidgets.QPushButton("Save")
        self.save_button.clicked.connect(self.save_settings)
        button_layout.addWidget(self.save_button)

        self.cancel_button = QtWidgets.QPushButton("Cancel")
        self.cancel_button.clicked.connect(self.reject)
        button_layout.addWidget(self.cancel_button)

        layout.addLayout(button_layout)

    def save_settings(self):
        data_mode = self.data_mode_dropdown.currentData()
        simulate_slowdown = self.slowdown_checkbox.isChecked()
        try:
            self.logger.debug("Saving user configuration")
            config = UserConfig()
            config.transaction_level = data_mode
            config.simulate_slowdown = simulate_slowdown
            self.accept()
        except TypeError as e:
            self.logger.error("Invalid user configuration found")
            QMessageBox.critical(None, "Invalid config detected", "Double check your configuration", QMessageBox.StandardButton.Ok, QMessageBox.StandardBUttons.NoButton)

